/*
 *				**** ALBUM TRACKS ****
 *
 * This file contains routines to read album tracks
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "alblist.h"
#include "bld_html.h"

struct alb_det *create_alb_det(void);
void bld_album_detail(FILE *fp, struct album *alb, struct alb_det *det);

void
read_alb_tracks(char *prog, struct album *alb)
{
	char buf[BUFSIZ], *songkey, *p, *q;
   struct alb_det *det=NULL, *det2;
   struct track *track;
   FILE *fp;	/* track info file */

   if (alb == NULL)
   {
   	fprintf(stderr, "%s: read_alb_tracks called with NULL arg!\n", prog);
      return;
   }

   sprintf(buf, "tracks/%s.txt", alb->key);
	if ((fp = fopen(buf, "r")) == NULL)
   {
   	/*****************************************
   	fprintf(stderr, "%s: Can't open track file (%s)\n", prog, buf);
      ****************************************/
      return;
   }

   while (read_line(buf, fp) == 0)
   { /* loop reading lines from the file */
   	if (isspace(*buf))
      {
      	if ((track = parse_track(buf)) == NULL)
         {
         	fprintf(stderr, "%s: Invalid track entry (%s) for album %s\n",
            	prog, buf, alb->title);
            continue;
         }
         if (det == NULL)
         {
         	fprintf(stderr, "%s: Track entry (%s) w/o album (%s)\n",
            	prog, track->title, alb->title);
            continue;
         }
			add_ptr_prodlist(det->tracks, track);
         /* check each title to see if we are interested */
         strcpy(buf, track->title); /* copy of entire title */
         for (p=buf; *p != '\0'; p = q)
         { /* check for multiple titles */
         	for (q=p; *q != '\0' && *q != '~'; q++);
            if (*q != '\0') *q++ = '\0'; /* terminate title */
         	songkey = get_song_key_for_title(p);
         	if (songkey != NULL)
	         { /* we are interested in this song */
         		add_album_to_song(alb->key, songkey); /* add to list */
         	}
         }
      }
      else
      { /* album record */
      	if ((det = parse_album(buf)) == NULL)
         {
         	fprintf(stderr, "%s: Invalid album entry (%s) for album (%s)\n",
            	prog, buf, alb->title);
            continue;
         }
         /* add to album */
         if (alb->det == NULL) alb->det = det;
         else
         { /* already one there - add to end of list */
         	for (det2=alb->det; det2->next != NULL; det2 = det2->next);
         	det2->next = det;	/* add to end */
         }
      }
   }

   fclose(fp);
}

struct alb_det *
create_alb_det(void)
{
	struct alb_det *ret;

   ret = (struct alb_det *)malloc(sizeof(struct alb_det));
   if (ret != NULL)
   {
   	memset(ret, '\0', sizeof(struct alb_det));
      ret->tracks = create_ptr_prodlist(15);
   }
   return ret;
}

struct track *
create_track(void)
{
	struct track *track;

   track = (struct track *)malloc(sizeof(struct track));
   if (track != NULL) memset(track, '\0', sizeof(struct track ));
   return track;
}

struct alb_det *
parse_album(char *str)
{
   char *flds[6];
   int nflds;
   struct alb_det *det;

   nflds = split_flds(str, flds, '|', 0, 6);
   if (nflds != 5) return NULL;

   /* album line looks fine - copy fields */
   if ((det = create_alb_det()) == NULL) return NULL;
   det->artist = strdup(flds[0]);
   det->title = strdup(flds[1]);
   det->date = strdup(flds[2]);
   det->label = strdup(flds[3]);
   det->catno = strdup(flds[4]);

   return det;
}

struct track *
parse_track(char *str)
{
	char *flds[4];
   int nflds;
   struct track *track;

   while (isspace(*str))str++; /* strip leading white space */
   nflds = split_flds(str, flds, '|', 0, 4);
   if (nflds < 0 || nflds > 3) return NULL;

   if ((track = create_track()) == NULL) return NULL;

   /* track line is okay - copy fields */
   track->title = strdup(flds[0]);
   if (nflds > 1) track->time = strdup(flds[1]);
   if (nflds > 2) track->artist = strdup(flds[2]);

   return track;
}

void
bld_alb_det(FILE *fp, char *key)
{
	struct album *alb;
   struct alb_det *det;

   alb = find_album_by_key(key);
   if (alb == NULL) return;

   for (det = alb->det; det != NULL; det = det->next)
   {
   	bld_album_detail(fp, alb, det);
   }
}

void
bld_album_detail(FILE *fp, struct album *alb, struct alb_det *det)
{
	int i;
   struct track *track;
   char *key, *p, *q, *str;

	/* description of album */
   do_include(fp, "AlbDet");
   do_include(fp, "AlbArtist");
   fprintf(fp, "%s\n", det->artist);
   do_include(fp, "AlbTitle");
   fprintf(fp, "%s\n", det->title);
   do_include(fp, "AlbDate");
   fprintf(fp, "%s\n", det->date);
   do_include(fp, "AlbLabel");
   fprintf(fp, "%s\n", det->label);
   do_include(fp, "AlbCatNo");
   fprintf(fp, "%s\n", det->catno);
   do_include(fp, "AlbDetEnd");

   /* now give track listing */
   do_include(fp, "AlbTracks");
   for (i=0; i < det->tracks->nitems; i++)
   {
   	track = (struct track *)det->tracks->item[i];
      do_include(fp, "TrackNum\n");
      fprintf(fp, "%d\n", i+1);
      do_include(fp, "TrackTitle");
      if (strchr(track->title, '~') == NULL)
		{ /* single title (normal case) */
	      if ((key = get_song_key_for_title(track->title)) != NULL)
	      {
	      	fprintf(fp, "<a href=../songs/%s.html#beg><font color=#ff0000><strong>\n", key);
	      }
	      fprintf(fp, "%s\n", track->title);
	      if (key != NULL) fputs("</strong></font></a>\n", fp);
      }
      else
      { /* more than one title - take apart */
      	str = strdup(track->title);
         for (p=str; *p != '\0'; )
         { /* find tilde or end of string */
         	for (q=p; *q != '\0' && *q != '~'; q++);
            if (*q != '\0') *q++ = '\0';
            if ((key = get_song_key_for_title(p)) != NULL)
            { /* insert link to song file */
            	fprintf(fp, "<a href=../songs/%s.html#beg><font color=#ff0000><strong>\n", key);
            }
            fprintf(fp, "%s\n", p);
            if (key != NULL) fputs("</strong></font></a>\n", fp);
            p = q;
            if (*p != '\0') fputs("<br>", fp);
         }
         free(str);	/* free memory */
      }
      do_include(fp, "TrackTime\n");
      fprintf(fp, "%s\n", track->time);
      do_include(fp, "TrackArtist\n");
      for (p=track->artist; p != NULL && *p != '\0'; p++)
      { /* write out artist(s) */
      	if (*p == '~') fputs("<br>", fp);
         else fputc(*p, fp);
      }
      do_include(fp, "TrackEnd");
   }
   do_include(fp, "AlbTracksEnd");
}

