/*
 *				**** BLD HTML ****
 *
 * This file contains routines that read an HTML file
 * and includes files as required
 *
 * Include lines begin with greater than (>) and
 * have the base name of the file following the >
 * The actual file name ends with .txt
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "bld_html.h"

static ninc = 0;		/* number of includes defined */
static minc = MAX_INC;	/* initial size of index array */
struct html_ndx *ndx=NULL;	/* array of includes */

static int verbose = 0;

void set_verbose_inc(int val) { verbose = val; }

static void
init_ndx(void)
{ /* initialize the index structure */
	if (verbose>9) printf("Initializing ndx\n");

	ndx = (struct html_ndx *)malloc(sizeof(struct html_ndx) * minc);
	if (ndx == NULL)
   {
   	fprintf(stderr, "Can't malloc memory for HTML index!\n");
      exit(1);
   }
}

static void
resize_ndx(void)
{
	fprintf(stderr, "Can't resize HTML index yet!\n");
   exit(1);
}

static int
find_inc(char *name)
{
	int i;

   for (i=0; i < ninc; i++)
   { /* search for match */
   	if (strcmpi(name, ndx[i].key) == 0) return i;
   }
   return -1;
}

static int
parse_inc(FILE *fp, char *name)
{ /* include started, find end & add entry */
	int ret = 0, n;	/* normal return code */
   char buf[BUFSIZ], str[INCSIZ];
   char *p;

   if (verbose>9) printf("Parsing include file (%s)\n", name);

   if (ninc == 0) init_ndx();
	if (ninc >= minc) resize_ndx();

   /* set up next entry, then find end of entry */
   ndx[ninc].key = strdup(name+1);	/* name of inc */
	*str = '\0';	/* clear include string */

   while (1)
   { /* read until we find another > */
      if (fgets(buf, BUFSIZ, fp) == NULL)
      { /* EOF - shouldn't happen */
      	ret = 1;	/* signal end of file */
         break;
      }
      if (*buf == '>')
      { /* either end or another include */
      	if (*(buf+1) == '\n' || *(buf+1) == '\0'
         	|| *(buf+1) == '\r')
         { /* end of this include */
         	break;
         }
         /* this is a embedded include */
         /* strip CR-LF */
         for (p=buf+1; *p != '\n' && *p != '\0'
         	&& *p != '\r'; p++);
			*p = '\0'; /* strip CR-LF */
         n = find_inc(buf+1);
         if (n < 0)
         {
         	fprintf(stderr, "Invalid Unknown include (%s)\n",
            	buf);
         }
         else
         { /* valid include - use it */
         	strcat(str, ndx[n].data);
         }
         continue; /* done append include command */
      }
      strcat(str, buf);		/* add string to end */
   }

   /* we found the end of the include */
   ndx[ninc++].data = strdup(str); /* include text */

   return ret;
}

void
do_include(FILE *outf, char *name)
{ /* execute include 'name' */
	int n;
   char *p;

   /* strip CR-LF from line */
   for (p=name; *p != '\0' && *p != '\n' && *p != '\r'; p++);
   *p = '\0';

   if ((n = find_inc(name)) < 0)
   {
   	fprintf(stderr, "unknown include (%s)\n", name);
      return;	/* continue */
   }

   /* include entry found in location 'n' */
   fputs(ndx[n].data, outf);	/* insert text */
}

void
bld_html(FILE *outf, FILE *inf)
{ /* build HTML file from input & includes */
	char buf[BUFSIZ];

   while (1)
   {
   	if (fgets(buf, BUFSIZ, inf) == NULL) break;	/* EOF */
      if (*buf == '>')
      { /* include file found - process it */
      	do_include(outf, buf+1); /* EOF */
      } else
      { /* regular line - output it */
      	fputs(buf, outf);
      }
   }
}

void
bld_inc(char *fname)
{ /* parse include file */
	FILE *fp;
   char buf[BUFSIZ];

   if ((fp = fopen(fname, "r")) == NULL)
   { /* can't open - ignore */
   	fprintf(stderr, "Can't open include file (%s)\n",
      	fname);
      return;
   }

   while (1)
   { /* read  until EOF */
   	if (read_line(buf, fp)) break; /* EOF */
      if (*buf == '>')
      { /* start of include */
	      if (parse_inc(fp, buf)) break; /* EOF */
      }
   }
   fclose(fp);	/* close file */
}

int
read_line(char *buf, FILE *fp)
{ /* read line & strip CR-LF */
	/* read line - non-zero return means EOF */
	if (fgets(buf, BUFSIZ, fp) == NULL) return 1;

   while (*buf != '\r' && *buf != '\n'
   	&& *buf != '\0') buf++;
   *buf = '\0';	/* truncate line */

   return 0;	/* normal return */
}

