/*
 *				**** BLD SONG ****
 *
 * This program will build song HTML files
 *
 * This program reads the basic page from the sng
 * directory and adds track info from the tracks
 * directory.
 *
 * The program will also include files for all
 * lines starting with greater than sign (>)
 * The text following the > is the include name
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "bld_html.h"
#include "alblist.h"

char *Prog = NULL;	/* program name */
int debugsw = 0;		/* debug switch */
int verbose = 0;		/* verbose switch */

void do_help(char *prog);
void do_usage(char *prog);
void do_msg(char *msg[], char *prog);

int
main(int argc, char *argv[])
{
	char buf[BUFSIZ], *p;

   	p = strrchr(argv[0], '/');
	if (p == NULL) p = strrchr(argv[0], '\\');
   	if (p == NULL) p = argv[0];
   	else p++; /* skip past last directory slash */

	Prog = p;	/* save program name */
   /* convert name to lower case (for DOS) */
   while (*p != '\0')
   { /* loop through program name */
   	if (isupper(*p)) *p = tolower(*p);
      p++;
   }
   argc--; argv++;	/* skip past program name */

   if (argc == 0)
   {
   	do_usage(Prog);
      exit(0);
   }


   set_verbose_inc(verbose); /* set verbose flag */

   /* read include files */
   bld_inc("include/include.txt");
   bld_inc("include/songinc.txt");

   /* ingest alblist and songlist */
   alblist(Prog);
   songlist(Prog);
   set_verbose_alblist(verbose);
   set_verbose_songlist(verbose);

   /* parse the include files */
   bld_inc("include/songinc.txt");
   bld_inc("include/include.txt");

   while (argc > 0)
   { /* process arguments */
   	if (argv[0][0] == '-') switch(argv[0][1])
      { /* process option */
      	case 'a': build_all_songs(); break;
      	case 'h': do_help(Prog); exit(0);
         case 'd': debugsw = !debugsw; break;
         case 'v':
         	if (argv[0][2] == '\0')
            { /* verbose value is next arg */
            	argc--; argv++;
               p = argv[0];
            } else p = argv[0]+2;
         	verbose = atoi(p);	/* set verbose flag */
            set_verbose_inc(verbose);
   			set_verbose_alblist(verbose);
   			set_verbose_songlist(verbose);
         	break;
         default:
         	fprintf(stderr, "%s: Invalid option (%s)\n",
            	Prog, argv[0]);
            do_usage(Prog);
            exit(1);
      } else
      { /* not option - must be file name */
      	if (verbose>1) printf("%s: Processing file %s ...\n",
         	Prog, argv[0]);
         strcpy(buf, argv[0]);	/* copy name */

         /* strip .html if there */
         if ((p = strstr(argv[0], ".htm")) != NULL)
         { /* found .htm or .html - get rid of it */
         	*p = '\0';
         }

         /* strip directory name if there */
         p = strrchr(argv[0], '/');
         if (p == NULL) p = strrchr(argv[0], '\\');
         if (p == NULL) p = buf;
         else p++;	/* skip past final slash */
         build_song(p);	/* build song HTML file */
      }
   	argc--; argv++;	/* done with argument */
   }

   printf("%s: Terminating\n", Prog);
   exit(0);
}

void
do_help(char *prog)
{
	char *msg[] = {
"This program builds song HTML files\n",
"Program arguments are the basename of the files\n",
"to be processed\n\n",
"Main files come from directory sng\n",
"Tracks files come from directory sngrec\n",
"Include files come from directory include\n",
"Output files go to directory songs\n",
NULL};

	do_msg(msg, prog);
}

void
do_usage(char *prog)
{
	char *msg[] = {
"Usage:\t%s [opts] files ...\n",
"-OR- for help, type:\n",
"\t%s -h\n",
"Where opts are:\n",
"\t-v val\tset verbose switch to 'val'\n",
"\t-d\ttoggle debug mode\n",
NULL};
	do_msg(msg, prog);
}

void
do_msg(char *msg[], char *prog)
{
	int i;

   for (i=0; msg[i] != NULL; i++)
   {
   	printf(msg[i], prog);
   }
}
