/*
 *				**** MAKE NEW ****
 *
 * This program will read a discography file,
 * a list of songs and albums, and compare the
 * information with existing data files to determine what
 * has been added.
 *
 * It will then update the necessary information,
 * maintaining a log of its activities
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "prodlist.h"
#include "alblist.h"
#include "bld_html.h"

char *Help[] = {
"This program will read a list of songs and albums\n",
"and what songs are on what albums.  It will then compare\n",
"this information with existing data files to determine what\n",
"has been added.\n",
"\n",
"It will then update the necessary information,\n",
"maintaining a log of its activities\n",
NULL};

char *Usage[] = {
"Usage:\t%s [options]\n",
"where options are:\n",
"\t-h\t\tDisplay help message\n",
"\t-v val\tSet verbose switch to 'val'\n",
NULL};


char *Prog = NULL;	/* program name */
int debugsw = 0;		/* debug switch */
int verbose = 9;		/* verbose switch */

void make_new(char *prog, int all_albs, int all_songs);

int
main(int argc, char *argv[])
{
	char buf[BUFSIZ], *p;
    int all_albs=0, all_songs=0;

   p = strrchr(argv[0], '/');
   if (p == NULL) p = strrchr(argv[0], '\\');
   if (p == NULL) p = argv[0];
   else p++; /* skip past last directory slash */

	Prog = p;	/* save program name */
   /* convert name to lower case & drop .exe (for DOS) */
   while (*p != '\0')
   { /* loop through program name */
   	if (stricmp(p, ".exe") == 0)
      { /* found .EXE (dos) */
      	*p = '\0';
         break;
      }
   	if (isupper(*p)) *p = tolower(*p);
      p++;
   }
   argc--; argv++;	/* skip past program name */

   /* read include files */
   bld_inc("include/include.txt");
   bld_inc("include/albinc.txt");
   bld_inc("include/songinc.txt");

   /* ingest alblist and songlist */
   alblist(Prog);
   songlist(Prog);
   set_verbose_alblist(verbose);
   set_verbose_songlist(verbose);

   while (argc > 0)
   { /* process arguments */
   	if (argv[0][0] == '-') switch(argv[0][1])
      { /* process option */
        case 'a': /* all albs and/or songs */
            if (argv[0][2] == 'a') all_albs++;
            else if (argv[0][2] == 's') all_songs++;
            else all_albs = all_songs = 1;
            break;
      	case 'h': do_msg(Usage, Prog); do_msg(Help, Prog); exit(0);
         case 'v':
         	if (argv[0][2] == '\0')
            { /* verbose value is next arg */
            	argc--; argv++;
               p = argv[0];
            } else p = argv[0]+2;
         	verbose = atoi(p);	/* set verbose flag */
            set_verbose_alblist(verbose);
            set_verbose_songlist(verbose);
         	break;
         default:
         	fprintf(stderr, "%s: Invalid option (%s)\n",
            	Prog, argv[0]);
            do_msg(Usage, Prog);
            exit(1);
      } else
      { /* not option - must be file name */
      	fprintf(stderr, "%s: File arguments ignored (%s)\n",
         	Prog, argv[0]);
      }
   	argc--; argv++;	/* done with argument */

   }

   make_new(Prog, all_albs, all_songs);

   printf("%s: Terminating\n", Prog);
   exit(0);
}

void
make_new(char *prog, int all_albs, int all_songs)
{
	char **albs, **songs, **songkeys;
   int nalb, nsong;
   int i, j;
   char *key;
   struct album *alb;
   struct alb_det *det;
   struct track *track;
   struct song *song;
   char buf[BUFSIZ];

   /* get list of album & song titles */
   nalb = get_number_albums();
   albs = get_album_titles();
   nsong = get_number_songs();
   songs = get_song_titles();

   printf("%d albums, %d songs, reading tracks ...\n",
   	nalb, nsong);

	alb_tracks(prog);	/* read track files */

   if (all_albs) build_all_albums();
   else update_album_files(prog); /* update changed files */

   song_albums(prog); /* read current list of songs & albums */

   if (all_songs) build_all_songs();
   else update_song_files(prog); /* update changed files */

   write_song_alb();		/* save song/album info */
}

